@@ -1,6 +1,7 @@
 import json
 import logging
 
+import borgmatic.actions.arguments
 import borgmatic.borg.list
 import borgmatic.config.validate
 
@@ -8,7 +9,13 @@ logger = logging.getLogger(__name__)
 
 
 def run_list(
-    repository, storage, local_borg_version, list_arguments, local_path, remote_path,
+    repository,
+    config,
+    local_borg_version,
+    list_arguments,
+    global_arguments,
+    local_path,
+    remote_path,
 ):
     '''
     Run the "list" action for the given repository and archive.
@@ -20,24 +27,27 @@ def run_list(
     ):
         if not list_arguments.json:  # pragma: nocover
             if list_arguments.find_paths:
-                logger.answer(f'{repository}: Searching archives')
+                logger.answer(f'{repository.get("label", repository["path"])}: Searching archives')
             elif not list_arguments.archive:
-                logger.answer(f'{repository}: Listing archives')
-        list_arguments.archive = borgmatic.borg.rlist.resolve_archive_name(
-            repository,
+                logger.answer(f'{repository.get("label", repository["path"])}: Listing archives')
+
+        archive_name = borgmatic.borg.rlist.resolve_archive_name(
+            repository['path'],
             list_arguments.archive,
-            storage,
+            config,
             local_borg_version,
+            global_arguments,
             local_path,
             remote_path,
         )
         json_output = borgmatic.borg.list.list_archive(
-            repository,
-            storage,
+            repository['path'],
+            config,
             local_borg_version,
-            list_arguments=list_arguments,
-            local_path=local_path,
-            remote_path=remote_path,
+            borgmatic.actions.arguments.update_arguments(list_arguments, archive=archive_name),
+            global_arguments,
+            local_path,
+            remote_path,
         )
         if json_output:  # pragma: nocover
             yield json.loads(json_output)
